/* 
	This's the C code to fill up the maze... The executable needs an
	input parameter equal to the name of the output flatfile for the
	maze... else it creates the file "mazefile". The output file is 32K
	in size and includes the 16K for the maze and the 16K for the crumb
	area. The byte number (0XY) gives the wall information about the
	point (X,Y) on the grid and (1XY) the corresponding point in the
	crumb file.
	This file is compilable/executable both on Windows (VC++5.0) and
	Linux (gcc/g++). 
*/


#include <stdio.h>
#include <stdlib.h>

#define MAX_ARRAY_SIZE 128 
#define WALL 0x40
#define NO_WALL 0x01
#define NORTH	0x02
#define WEST	0x04
#define SOUTH	0x08
#define EAST	0x10
#define EXIT	0x20
#define CRUMB 	0XFF

#define FILENAME "mazefile"

void make_a_wall(int ll_x, int ll_y, int width, int height); 
void mark_points_in_between();
void write_to_file(char * name);

char maze[MAX_ARRAY_SIZE][MAX_ARRAY_SIZE];

int  main(int argc, char ** argv){
	int i,j;
	for(i = 0; i<MAX_ARRAY_SIZE; i++){
		for(j=0; j<MAX_ARRAY_SIZE; j++){
			maze[i][j] = (char) 0x00;
		}
	}

    	make_a_wall(0,0,1,MAX_ARRAY_SIZE);
	make_a_wall(1,0,MAX_ARRAY_SIZE-1,1);
	make_a_wall(MAX_ARRAY_SIZE-1,1,1,MAX_ARRAY_SIZE-1);
	make_a_wall(1,MAX_ARRAY_SIZE-1,MAX_ARRAY_SIZE-2,1);

	make_a_wall(4,1,11,60);
	make_a_wall(32,1,11,32);
	make_a_wall(43,1,30,8);
	make_a_wall(84,1,9,24);
	make_a_wall(93,16,34,9);
	make_a_wall(96,48,31,9);
	make_a_wall(104,64,13,17);
	make_a_wall(92,92,9,9);
	make_a_wall(101,96,26,5);
	make_a_wall(8,76,81,9);
	make_a_wall(40,56,9,71);
	make_a_wall(49,40,40,9);
	make_a_wall(64,48,25,28);
	make_a_wall(49,104,40,9);
	make_a_wall(80,113,9,8);
	make_a_wall(8,92,13,35);
	make_a_wall(21,116,19,11);

	mark_points_in_between();

	maze[60][126] = (char) EXIT;

	write_to_file((argc>1)?argv[1]:FILENAME);

	return 0;
}


void make_a_wall(int ll_x, int ll_y, int width, int height){ 
    int i,j;
    for(i = ll_x; i<ll_x+width; i++){
	for(j = ll_y; j<ll_y+height; j++){
	    maze[i][j] = (char) WALL;
	}
    }
    return;
}


void mark_points_in_between(){
    int i,j;
    char wall_status;
    for(i=1; i<MAX_ARRAY_SIZE-1; i++){
        for(j=1; j<MAX_ARRAY_SIZE-1; j++){
	    if( maze[i][j] == (char) WALL) continue;
	    wall_status = (char) 0x0;
	    wall_status |= (maze[i][j-1]==(char)WALL)?(char)SOUTH:wall_status;
	    wall_status |= (maze[i-1][j]==(char)WALL)?(char)WEST:wall_status;
	    wall_status |= (maze[i][j+1]==(char)WALL)?(char)NORTH:wall_status;
	    wall_status |= (maze[i+1][j]==(char)WALL)?(char)EAST:wall_status;
	    if (!wall_status) wall_status = (char)NO_WALL; 
	    maze[i][j] = wall_status;
	}
    }
    return;
}

void write_to_file(char * name){
    int i,j;
    FILE * fp = fopen(name,"w");

    for(i=0; i<MAX_ARRAY_SIZE; i++){
    	for(j=0; j<MAX_ARRAY_SIZE; j++){
	    fputc(maze[i][j],fp);
	}
    }

    for(i=0; i<MAX_ARRAY_SIZE; i++){
    	for(j=0; j<MAX_ARRAY_SIZE; j++){
	    fputc((maze[i][j]==(char)WALL)?(char)WALL:(char)CRUMB,fp);
	}
    }

    fclose(fp);
}
